<H3>Instruction Set (ISA.lhs)</H3>

\begin{code}
module ISA where

import Ix

\end{code}
The instruction set modelled
in this example is extremely simple. 
In this simple ISA, only allow some integer ALU instructions and
memory loads and stores (plus a Nop instruction) .
For instance,
there are no jump instructions or even the notion of a program
counter. Instead the pipeline accepts a linear list of instructions,
one per clock cycle, and executes each instruction in turn.
The instructions that the pipeline does support are:
<P>
<TABLE BORDER=0>
<TR>
<TD><b>Nop</b></TD>
<TD>No operation</TD>
</TR>
<TR>
<TD><b>LdImm</b></TD>
<TD>Load a constant integer into a register</TD>
</TR>
<TR>
<TD><b>Add</b></TD>
<TD>Add two registers and store the
the result into a third register</TD>
</TR>
<TR>
<TD><b>Sub</b></TD>
<TD>Subtract two registers and store the
the result into a third register</TD>
</TR>
<TR>
<TD><b>Mul</b></TD>
<TD>Multiply two registers and store the
the result into a third register</TD>
</TR>
<TR>
<TD><b>Load</b></TD>
<TD>Load an integer stored at a memory
address specified by one register
into a second register
</TD>
</TR>
<TR>
<TD><b>Store</b></TD>
<TD>Store the contents of a register into
the memory address specified by another
register</TD>
</TR>
</TABLE>
<P>
The 'LdImm' instruction has an extra RegValue field.
This instruction assigns that value to the destination
register, ignoring the contents of the other source operands.
<P>
Type of register names. In this architecture, R0 is
a constant-valued register, always set to zero.
\begin{code}
data RegName = R0 | R1 | R2 | R3 | R4 | R5 | R6 | R7
               deriving (Read,Show,Eq,Ord,Ix,Enum)
\end{code}

\begin{code}
type RegValue = Integer
\end{code}


\begin{code}
data Opcode     = Nop
                | LdImm RegValue
                | Add
                | Sub
                | Mul 
                | Load
                | Store
                deriving (Read,Show,Eq)
\end{code}
