>{-# OPTIONS -fglasgow-exts #-}

> module StaticArrow(Cartesian(..), StaticArrow, StaticMonadArrow,
>		wrapM, unwrapM) where

> import Arrow
> import ArrowTransformer
> import ArrowOps
> import Monad

> class Functor f => Cartesian f where
>	unit :: a -> f a
>	prod :: f a -> f b -> f (a,b)

> newtype StaticArrow f a b c = SA (f (a b c))

> instance (Cartesian f, Arrow a) => Arrow (StaticArrow f a) where
>	arr f = SA (unit (arr f))
>	SA f >>> SA g = SA (fmap (uncurry (>>>)) (f `prod` g))
>	first (SA f) = SA (fmap first f)

> instance (Cartesian f, Arrow a) => ArrowTransformer (StaticArrow f) a where
>	lift f = SA (unit f)

> instance Functor f => ArrowFunctor (StaticArrow f) where
>	arrowmap embed (SA f) = SA (fmap embed f)

The following promotions follow directly from the arrow transformer.

> instance (ArrowZero a, Cartesian f) =>
>		ArrowZero (StaticArrow f a) where
>	zeroArrow = lift zeroArrow

> instance (ArrowCircuit a, Cartesian f) =>
>		ArrowCircuit (StaticArrow f a) where
>	delay x = lift (delay x)

> instance (ArrowExcept ex a, Cartesian f) =>
>		ArrowExcept ex (StaticArrow f a) where
>       raise = lift raise

> instance (ArrowReader r a, Cartesian f) =>
>		ArrowReader r (StaticArrow f a) where
>       readState = lift readState

> instance (ArrowState s a, Cartesian f) =>
>		ArrowState s (StaticArrow f a) where
>       fetch = lift fetch
>       store = lift store

> instance (ArrowWriter w a, Cartesian f) =>
>		ArrowWriter w (StaticArrow f a) where
>       write = lift write

Classes that are preserved.

> instance (Cartesian f, ArrowChoice a) => ArrowChoice (StaticArrow f a) where
>	left (SA f) = SA (fmap left f)

ArrowApply is generally not preserved.

> instance (Cartesian f, ArrowLoop a) => ArrowLoop (StaticArrow f a) where
>	loop (SA f) = SA (fmap loop f)

> instance (Cartesian f, ArrowPlus a) => ArrowPlus (StaticArrow f a) where
>       SA f <+> SA g = SA (fmap (uncurry (<+>)) (f `prod` g))

promotions

> instance (Cartesian f, ArrowRunState s a a') =>
>		ArrowRunState s (StaticArrow f a) (StaticArrow f a') where
>	runState (SA f) = SA (fmap runState f)

> instance (Cartesian f, ArrowRunReader r a a') =>
>		ArrowRunReader r (StaticArrow f a) (StaticArrow f a') where
>	runReader (SA f) = SA (fmap runReader f)

> instance (Cartesian f, ArrowNewReader r a) =>
>		ArrowNewReader r (StaticArrow f a) where
>	newReader (SA f) = SA (fmap newReader f)

> instance (Cartesian f, ArrowRunWriter w a a') =>
>		ArrowRunWriter w (StaticArrow f a) (StaticArrow f a') where
>	runWriter (SA f) = SA (fmap runWriter f)

> instance (Cartesian f, ArrowNewWriter w a) =>
>		ArrowNewWriter w (StaticArrow f a) where
>	newWriter (SA f) = SA (fmap newWriter f)

> instance (Cartesian f, ArrowRunExcept ex a a') =>
>		ArrowRunExcept ex (StaticArrow f a) (StaticArrow f a') where
>	runExcept (SA f) (SA h) = SA (fmap (uncurry runExcept) (f `prod` h))

> instance (Cartesian f, ArrowNewExcept ex a) =>
>		ArrowNewExcept ex (StaticArrow f a) where
>	newExcept (SA f) (SA h) = SA (fmap (uncurry newExcept) (f `prod` h))

> instance (Cartesian f, ArrowRunMaybe a a') =>
>		ArrowRunMaybe (StaticArrow f a) (StaticArrow f a') where
>	runMaybe (SA f) (SA h) = SA (fmap (uncurry runMaybe) (f `prod` h))

The following is a special case:

> newtype CartesianArrow a b c = CA (a b c)

> instance Arrow a => Functor (CartesianArrow a s) where
>	fmap k (CA f) = CA (f >>> arr k)

> instance Arrow a => Cartesian (CartesianArrow a s) where
>	unit x = CA (arr (const x))
>	CA f `prod` CA g = CA (f &&& g)

> type StaticArrowArrow a s = StaticArrow (CartesianArrow a s)

> wrapA :: (Arrow a, Arrow a') => a s (a' b c) -> StaticArrowArrow a s a' b c
> wrapA x = SA (CA x)

> unwrapA :: (Arrow a, Arrow a') => StaticArrowArrow a s a' b c -> a s (a' b c)
> unwrapA (SA (CA x)) = x

A special case of this is monads:

> newtype CartesianMonad m a = CM (m a)

> instance Monad m => Functor (CartesianMonad m) where
>	fmap k (CM f) = CM (liftM k f)

> instance Monad m => Cartesian (CartesianMonad m) where
>	unit x = CM (return x)
>	CM u `prod` CM v = CM (u >>= \x -> v >>= \y -> return (x,y))

> type StaticMonadArrow m = StaticArrow (CartesianMonad m)

> wrapM :: (Monad m, Arrow a) => m (a b c) -> StaticMonadArrow m a b c
> wrapM x = SA (CM x)

> unwrapM :: (Monad m, Arrow a) => StaticMonadArrow m a b c -> m (a b c)
> unwrapM (SA (CM x)) = x
