-- Fun with masks

module Masks where

import Monad
import Pan


-- 1D wipe.  0 for x<-.5, 1 for x>.5, and linear in-between
wipe :: FloatE -> FractionE
wipe x = ifE (x <* -0.5) 0 (ifE (x <* 0.5) (0.5 + x) 1)

-- Some makers of 2D wipes, for soft combination lerping.
wipe2 :: FloatE -> Image FractionE
-- Shifts across Y axis
wipe2 w = uscale w (wipe . fst)

-- "Soft" disk with radius r, whose boundary has width w.  Has value 1
-- sufficiently far inwards and 0 sufficiently far outwards.
-- Question: should this radius be inner, outer, or middle?  Now middle.
softDisk r w p = wipe ((r - distO p) / w)

softCropUI :: UI FilterC
softCropUI = do r <- slider "crop radius" (0,200) 40
                w <- slider "softness" (0.01,50) 10
                return $ fade (softDisk r w)

-- Some animated regions
maskUI :: UI (TimeE -> Region)
maskUI = radioBox "mask"
          [ ("sweep" , sweepF), ("rotate" , rotateF)
          , ("swirl" , swirlF), ("hole" , holeF) ]
          1
 where
   sweepF, rotateF, swirlF, holeF :: TimeE -> Region
   sweepF  t = translate (75 * sin t, 0) xPos
   rotateF t = rotate t xPos
   swirlF  t = swirl (10 / sin t) xPos
   holeF   t = uscale 30 (translate (cos (1.5*t), sin (2*t)) udisk)



-- Use the masks to select between two images.
--hybridUI :: Syntactic a => UI (ImageCombiner a)
hybridUI :: UI ImageCombiner
hybridUI = do maskF <- maskUI
              return $ \ imA imB t -> cond (maskF t) imA imB

---- Soft masks and soft hybrids

-- Make a soft mask (fraction-valued image), given width and
-- time/progress.

softMaskUI :: UI (TimeE -> Image FractionE)
softMaskUI = ap (radioBox "mask"
                  [ ("sweep" , sweepF), ("rotate" , rotateF)
                  , ("swirl" , swirlF), ("hole" , holeF) ]
                  1)
                (slider "softness" (0.01,200) 50)
 where
   sweepF, rotateF, swirlF, holeF :: FloatE -> TimeE -> Image FractionE
   sweepF w t = translate (100 * sin t, 0) (wipe2 w)
   rotateF  w t = rotate t (wipe2 w)
   swirlF w t = swirl (10 / sin t) (wipe2 w)
   holeF  w t = translate (r*cos t, r*sin (2*t)) (softDisk r w)
                where r = 50


softHybridUI :: UI ImageCombiner
softHybridUI = do maskF <- softMaskUI
                  return $ \ imA imB t -> lerpI (maskF t) imA imB


----------------------------------------

makeAll = makeAllFor allBackends

makeAllFor bends = do
  compileFilterFor   bends "soft crop"   $ atMouse softCropUI
  compileImageBFor   bends "masks"       maskUI
  compileImageBFor   bends "soft masks"  softMaskUI
  compileCombinerFor bends "soft hybrid" softHybridUI
  compileCombinerFor bends "hybrid"      hybridUI
