
#ifndef __MAINWINDOW_H__
#define __MAINWINDOW_H__

#include "stdafx.h"
#include "PanFactory.h"
#include "PanUtils.h"
#include <vector>
using namespace std;

typedef CWinTraits <WS_VISIBLE | WS_OVERLAPPEDWINDOW> MainTraits;

//
// CLASS:   CPanMainWindow
//
// PURPOSE: Embed a Pan effect inside a window.
//
// COMMENTS: Use templates to parameterise the 

template<unsigned int MenuID = 101>
class CPanMainWindow : public CWindowImpl<CPanMainWindow,CWindow,MainTraits>
{
public:
    DECLARE_WND_CLASS_ICON(NULL,MenuID)

    CPanMainWindow() : m_hWndEffect(NULL), m_maxPasses(40), m_maxJitter(1.0) { }

    BEGIN_MSG_MAP(CMainWindow)
        MESSAGE_HANDLER(WM_SIZE, OnSize)
        MESSAGE_HANDLER(WM_CLOSE, OnClose)
        MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
    END_MSG_MAP()

    void OpenEffect(CComQIPtr<IPanImage> pPanObj, LPCTSTR effectName)
    {
        CComQIPtr<IPanImageUI> pUI;
        CWindowVirt*           winUI;
        CViewWindow*           pView;
        HRESULT                hr;
        HFONT                  hFont;
        RECT                   r;
        CComBSTR               docURL = "";

        vector<CStringProp*>* propVec = new vector<CStringProp*>();
        vector<CFontProp*>* fontVec   = new vector<CFontProp*>();

        hr = pPanObj->QueryInterface(&pUI);

        if (SUCCEEDED(hr)) {
            CPanFactory panFact(pPanObj,&m_maxPasses,&m_maxJitter, docURL);

            winUI = panFact.Create(m_hWnd,propVec,fontVec,NULL,&pView);

            // Use the same font throughout the UI.
            HDC hDCUI = GetDC();

            hFont =
                CreateFont(-MulDiv(8, GetDeviceCaps(hDCUI, LOGPIXELSY), 72),
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, VARIABLE_PITCH | FF_SWISS, TEXT(""));

            CPanUtils::BroadcastFont(*winUI, hFont);
            ReleaseDC(hDCUI);

            m_hWndEffect = *winUI;

            // Adjust the size of the main window to naturally fit the effect.
            ::GetClientRect(m_hWndEffect,&r);
            ::SetWindowPos(m_hWnd, HWND_TOP,  0, 0, r.right - r.left, r.bottom - r.top, SWP_NOMOVE);

            //CEffect* ef = new CEffect(winUI, img, propVec, fontVec, progID, pView, m_hWndMain);
        }
        return;
    }


   LRESULT OnSize(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {

        if (m_hWndEffect) {
            ::SetWindowPos(m_hWndEffect, HWND_TOP,  0, 0, LOWORD(lParam), HIWORD(lParam), SWP_NOMOVE); 
        }
        bHandled = false;
        return 0;
    }

   LRESULT OnClose(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
     DestroyWindow();
     return 0;
   }

   LRESULT OnDestroy(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
     ::PostQuitMessage(0);
     return 0;
   }

private:
    HWND m_hWndEffect;
    int  m_maxPasses;
    double m_maxJitter;
};

#endif /* __MAINWINDOW_H__ */
