//
// Rendering procedure for a Pan image.
//
// This code is conditionally included by PanImage.cpp
//
#ifdef USE_RENDER_THREAD


// 
// Data communicated by the viewer thread. Closures would
// have been nice here, so that the thread could just evaluate
// a thunk rather than having to buy into a fixed and cumbersome
// data structure like RenderData.
// 
class RenderData {
public:
  float t;
  int xStart, yStart;
  int rWidth, rHeight;
  float weight;
  int imageBuffer;
  CComQIPtr<IPanImage> disper;

  ~RenderData() { disper.Detach(); };
};


// structure holding the IPC handles used by the
// viewer to communicate with the render procedure.
typedef struct {
    HANDLE      hStart;
    HANDLE      hComplete;
    HANDLE      hQuit;
    RenderData  rData;
      // the RenderData is filled in by the viewer
      // prior to signalling the render procedure that
      // there's more work to do.
} ThreadData;


unsigned
__stdcall
RenderProc(void* argList)
{
  ThreadData* tData = (ThreadData*)argList;
  HANDLE hArr[2];
  DWORD dwRet;

  hArr[0] = tData->hStart;
  hArr[1] = tData->hQuit;

  _controlfp ( _MCW_RC, _RC_DOWN );
    // The generated Pan code assumes that the rounding mode is
    // set to 'round towards minus infinity'.


  while(1) {

    dwRet = WaitForMultipleObjects(2,hArr,FALSE,INFINITE);

    if ( dwRet != WAIT_OBJECT_0) { // viewer signalled shutdown on hQuit.
                                 // or any other
        break;
    }

    tData->rData.disper->Display
         ( tData->rData.t
		 , tData->rData.xStart
		 , tData->rData.yStart
		 , tData->rData.rWidth
		 , tData->rData.rHeight
		 , tData->rData.weight
		 , 0
		 , tData->rData.imageBuffer
		 );

    // signal that we've finished rendering the alloted portion.
    SetEvent(tData->hComplete);
  }

  delete argList;

  return 0;
}
#endif
