// Viewer.h : Declaration of the CViewer

#ifndef __VIEWER_H_
#define __VIEWER_H_

#include "resource.h"       // main symbols
#include <shlwapi.h>
#include <atlctl.h>
#include "PanViewerView.h"


/////////////////////////////////////////////////////////////////////////////
// CViewer
class ATL_NO_VTABLE CViewer : 
    public CComObjectRootEx<CComSingleThreadModel>,
    public IDispatchImpl<IViewer, &IID_IViewer, &LIBID_VIEWCONTROLLib>,
    public IObjectSafetyImpl<CViewer, INTERFACESAFE_FOR_UNTRUSTED_DATA | INTERFACESAFE_FOR_UNTRUSTED_DATA>,
    public CComControl<CViewer>,
    public IPersistStreamInitImpl<CViewer>,
    public IOleControlImpl<CViewer>,
    public IOleObjectImpl<CViewer>,
    public IOleInPlaceActiveObjectImpl<CViewer>,
    public IViewObjectExImpl<CViewer>,
    public IOleInPlaceObjectWindowlessImpl<CViewer>,
    public IPersistPropertyBagImpl<CViewer>,
    public CComCoClass<CViewer, &CLSID_Viewer>
{
public:
    CViewer()
        : m_hWndClient(NULL), m_maxPasses(40), m_maxJitter(0.7),
          m_view(&m_maxPasses,&m_maxJitter)
	{
            m_bWindowOnly = TRUE;
            m_bstrEffect = "";
	}

DECLARE_REGISTRY_RESOURCEID(IDR_VIEWER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_CATEGORY_MAP(CViewer)
    IMPLEMENTED_CATEGORY(CATID_Insertable)
    IMPLEMENTED_CATEGORY(CATID_Control)
    IMPLEMENTED_CATEGORY(CATID_SafeForScripting)
    IMPLEMENTED_CATEGORY(CATID_SafeForInitializing)
END_CATEGORY_MAP()

BEGIN_COM_MAP(CViewer)
    COM_INTERFACE_ENTRY(IViewer)
    COM_INTERFACE_ENTRY(IDispatch)
    COM_INTERFACE_ENTRY(IViewObjectEx)
    COM_INTERFACE_ENTRY(IViewObject2)
    COM_INTERFACE_ENTRY(IViewObject)
    COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
    COM_INTERFACE_ENTRY(IOleInPlaceObject)
    COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
    COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
    COM_INTERFACE_ENTRY(IOleControl)
    COM_INTERFACE_ENTRY(IOleObject)
    COM_INTERFACE_ENTRY(IPersistStreamInit)
    COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
    COM_INTERFACE_ENTRY(IPersistPropertyBag)
//    COM_INTERFACE_ENTRY(IObjectSafety)
END_COM_MAP()

BEGIN_PROP_MAP(CViewer)
    PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
    PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
    PROP_ENTRY("Effect", DISPID_EFFECT, CLSID_NULL)
    // Example entries
    // PROP_ENTRY("Property Description", dispid, clsid)
    // PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CViewer)
    MESSAGE_HANDLER(WM_CREATE, OnCreate)
    MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
    MESSAGE_HANDLER(WM_SIZE, OnSize)
    CHAIN_MSG_MAP(CComControl<CViewer>)
    DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IViewer
public:

    CComBSTR m_bstrEffect;
    CPanViewerView m_view;
    HWND m_hWndClient;

    HRESULT OnDraw(ATL_DRAWINFO& di)
    {
            RECT& rc = *(RECT*)di.prcBounds;
            Rectangle(di.hdcDraw, rc.left, rc.top, rc.right, rc.bottom);

            SetTextAlign(di.hdcDraw, TA_CENTER|TA_BASELINE);
            LPCTSTR pszText = _T("Pan Viewer Control 1.0");
            TextOut(di.hdcDraw, 
                    (rc.left + rc.right) / 2, 
                    (rc.top + rc.bottom) / 2, 
                    pszText, 
                    lstrlen(pszText));

            return S_OK;
    }

    STDMETHOD(GetExtent)(DWORD dwDrawAspect, SIZEL *psizel) {
        CViewer* pT = static_cast<CViewer*>(this);
        ATLTRACE2(atlTraceControls,2,_T("IOleObjectImpl::GetExtent\n"));
        if (dwDrawAspect != DVASPECT_CONTENT)
            return E_FAIL;
        if (psizel == NULL)
            return E_POINTER;

        // In order to compute the correct size of the control, instantiate
        // the component as a child of the desktop window. When the control
        // window is later created, we re-parent the entire UI tree & make the
        // window visible.
        if (m_hWndClient == NULL &&
            (m_sizeExtent.cx == m_sizeNatural.cx && m_sizeExtent.cy == m_sizeNatural.cy)) {
            // only instantiate the component if it hasn't already been *and* the user hasn't
            // supplied a custom size.
            InstantiateComponent(GetDesktopWindow(), FALSE);
            // the UI layer reports back the size in pixel; OLE uses HIMETRIC,
            // so convert it.
            AtlPixelToHiMetric (&m_sizeExtent, &m_sizeExtent);
        }
        *psizel = pT->m_sizeExtent;
        return S_OK;
    }

private:
    //
    // METHOD:  InstantiateComponent
    //
    // PURPOSE: Attempt to instantiate the Pan effect.
    //
    void InstantiateComponent(HWND hWnd, BOOL visible)
    {
        USES_CONVERSION;
        RECT rc;
        CLSID clsid;
        SIZE sz;
        LPTSTR name = TEXT("PanEffect");

        rc.top = rc.left = 0;
        rc.bottom = m_sizeExtent.cy;
        rc.right  = m_sizeExtent.cx;

        m_hWndClient = 
            m_view.Create(hWnd,
                          rc,
                          NULL,
                          WS_CHILD | (visible ? WS_VISIBLE  : 0) |
                          WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
                          WS_EX_CLIENTEDGE);

        if (SUCCEEDED(CLSIDFromProgID(m_bstrEffect, &clsid))) {
            sz = m_view.OpenEffect(m_hWndClient, clsid, name, NULL);
            // Unfortunately, it is a bit too late in the game to change the size
            // of the control now; the container won't pick it up. And there doesn't
            // appear to be a general mechanism for the control to ask the container
            // to resize either. Hence, the user of the control needs to manually
            // resize the container to fit.
            m_sizeExtent = sz;

        } else {
            // Not a ProgID, maybe it is a .pan file?

            // First see if it is available via the filesystem...
            m_view.SetDocumentURL(m_bstrEffect);
            sz = m_view.OpenEffect(m_hWndClient, OLE2T(m_bstrEffect), NULL, name);

            if (sz.cx == 0 && sz.cy == 0) {
                // Won't give up just yet! Check to see whether it is URL; if it
                // is, then download it to the cache & load it as a file.
                USES_CONVERSION;
                TCHAR effectURL[MAX_PATH];
                TCHAR cachedURL[MAX_PATH];
                DWORD dwLen = MAX_PATH;

                // requires at least Win98 (or Win95 with IE5)
                BOOL isValidURL = UrlIs(OLE2T(m_bstrEffect),URLIS_URL);

                if (isValidURL && SUCCEEDED(URLDownloadToCacheFile(NULL, OLE2T(m_bstrEffect), cachedURL, dwLen, 0, NULL)) ) {
                    // We want ImportImage to use the settings (.pan) file to resolve
                    // relative URLs.  ToDo: use a list of locations for import
                    // resolution, including: .pan file, embedding document, and others as
                    // chosen by the user (usually including Pan/import).  Give settings
                    // location priority over document, I think.
                    m_view.SetDocumentURL(m_bstrEffect);
                    // try using the absolute URL
                    sz = m_view.OpenEffect(m_hWndClient, cachedURL, NULL, name);
                } else if ( SUCCEEDED(UrlCombine(OLE2T(m_bstrURL), OLE2T(m_bstrEffect), effectURL, &dwLen,0)) ) {
                    // last-chance saloon, if the Effect is a relative URL, try combining
                    // it with the URL of the document the control appears in & try to load that.
                    dwLen = MAX_PATH;
                    if (SUCCEEDED(URLDownloadToCacheFile(NULL, effectURL, cachedURL, dwLen, 0, NULL))) {
                        // Try opening the cached file...
                        m_view.SetDocumentURL(effectURL);
                        sz = m_view.OpenEffect(m_hWndClient, cachedURL, NULL, name);
                    }
                }
            }
            // the UI layer reports back the size in pixel; OLE uses HIMETRIC,
            // so convert it.
            AtlPixelToHiMetric (&sz, &m_sizeExtent);
            m_sizeExtent = sz;
        }
    }

public:

    LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
    {
        if (m_hWndClient == NULL) {
            InstantiateComponent(m_hWnd, TRUE);
        } else {
            ::SetParent(m_hWndClient,m_hWnd);
            ::ShowWindow(m_hWndClient, SW_SHOW);
        }
        return 0;
    }

    LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
    {
        // So we know to re-instantiate the component.
        // ToDo: are we shutting down properly?  Do we need to 
        // to m_view.CloseEffect, etc?
        m_hWndClient = NULL;
        bHandled = FALSE;  // continue destroy handling.  (necessary?)
        return 0;
    }

    LRESULT OnSize(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
    {
        if (m_hWndClient) {
            ::SetWindowPos(m_hWndClient, HWND_TOP,  0, 0, LOWORD(lParam), HIWORD(lParam), SWP_NOMOVE); 
        }
        return 0;
    }



    STDMETHOD(put_Effect) (BSTR val)  
    {
        //delete m_bstrEffect;
        m_bstrEffect = ::SysAllocString(val);
        return S_OK;
        
    }

    STDMETHOD(get_Effect) (BSTR* pVal)
    {
        if (pVal) {
            *pVal = ::SysAllocString(m_bstrEffect);
            return S_OK;
        }
        return E_FAIL;
    }

    // To be able to resolve relative URLs in the Effect parameter
    // of the view control, we need to snaffle the URL of the document
    // the control appears in (if it is hosted by IE.)
    //
    // Info on how to do this was found in the KB article "HOWTO: Retrieve the URL
    // of a Web Page from an ActiveX Control" This technique also works for
    // other containers (such as Word, PP), provided you supply a binding context
    // to GetDisplayName().
    //
    STDMETHOD(SetClientSite)(IOleClientSite *pClientSite)
    {
        if (pClientSite != NULL) {
            // Obtain URL from container moniker.
            CComPtr<IMoniker> spmk;
            LPOLESTR pszDisplayName;
            CComPtr<IBindCtx> pbc;
            HRESULT hr;

            if (SUCCEEDED(pClientSite->GetMoniker(OLEGETMONIKER_TEMPFORUSER,
                                                  OLEWHICHMK_CONTAINER,
                                                  &spmk))) {
                if ( SUCCEEDED(CreateBindCtx(0, &pbc)) ) {
                    if (SUCCEEDED(hr = spmk->GetDisplayName(pbc, NULL, &pszDisplayName))) {
                        m_bstrURL = pszDisplayName;
                        // Communicate the URL to the class responsible for
                        // instantiating a component, so that the ImportImage functionality
                        // can make use of relative URLs.  If an effect is
                        // loaded directly by ProgId, this setting will
                        // stick.  If instead a settings (.pan) file is
                        // loaded, then its location will override the
                        // document's in resolving image import locations.
                        m_view.SetDocumentURL(m_bstrURL);
                        CoTaskMemFree((LPVOID)pszDisplayName);
                    }
                }
            }
        }
        return IOleObject_SetClientSite(pClientSite);
    }

private:
    int  m_maxPasses; double m_maxJitter; // for anti-aliasing
    CComBSTR m_bstrURL;

};

#endif //__VIEWER_H_
