%
%
%

This module provides various helper functions for generating
COM components encapsulating Pan effects/filters.

\begin{code}
module CompileComponent where

import Pretty
import PrettyUtils
import CompileDXT
import CCompile ( showCDoc, CCode )
import PanUtils ( unregisterProgID )
\end{code}

Generate the registration script for a DXTransform component.

\begin{code}
emitCompFiles :: String -> String -> CCode -> CCode -> IO ()
emitCompFiles name outDir fieldDecls ccode = do
   o' <- mkObjectInfo "Pan."
   let o = o'{className=name, vProgID= ("Pan."++)}
   writeFile (outDir ++ "\\PanImage.h2") 
             (showCDoc fieldDecls  ++ showCDoc ccode)
   writeFile (outDir ++ "\\PanImage.rgs") (genRegScript o False)
   writeFile (outDir ++ "\\config.h")
   	     (show
               (text "#define COMPONENT_CLSID" <+> text (clsid_uuid o)   $$
		text "#define COMPONENT_LIBID" <+> text (libid_uuid o)   $$
		text "#define CLSID_STRING"    <+> text (show (clsid o)) $$
		text ""))
    -- to recycle some registry space, we unregister a server/component that is
    -- mapped to the same ProgID (but we won't delete it.)
    -- Hmm..this unregisters before we know whether compilation has succeeded or not;
    -- which is arguably a little bit too early.
   unregisterProgID (encodeProgID (progID o (vProgID o name) name))
   -- putStrLn $ name ++ " classID: " ++ clsid_uuid o

\end{code}
