-- Discrete convolution with wrappers for continuous images

-- Work at discrete level.  Evaluate and expand kernel at run-time.
-- Compile-time generation based on code unrolling seems too expensive at
-- compile-time.

module Convolve (convolveCC,convolveDC,convolveDD, gauss) where

import IOExts (trace)

import Exp
import Color
import Image
import Arrays
import Bitmap

-- Continuous in and out images
convolveCC :: IntE -> (IntE,IntE) -> Image FloatE -> ImageC -> ImageC
convolveCC radius (iw,ih) kernel img =
  convolveDC radius kernel (pixelize (iw,ih) img)

-- Discrete input image and continuous kernel and out.  This one seems the
-- most useful in practice.  Presumes that the input array is efficiently
-- accessed.  If not, perform a reifyArray2 on it before convolving.
convolveDC :: IntE -> Image FloatE -> Array2 ColorE -> ImageC
convolveDC radius kernel imgD =
  reconstruct (arr2Map fromBGRA32 convolved)
 where
   convolved = reifyArray2 $ convolveDD radius kernelD imgD
   size      = (s,s)
   s         = 2 * radius + 1
   kernelD   = reifyArray2 $ pixelize size kernel

-- Oh!  This is just the fadeC function.  ToDo: use fadeC instead (and retest)
(*^) :: FloatE -> ColorE -> ColorE
c *^ (b,g,r,a) = --trace ("*^ " ++ show c) $
                 (f b, f g, f r, f a)
 where
   f = (toBF c `multBF`)

-- Discrete input image, kernel and out
convolveDD :: IntE -> Array2 FloatE -> Array2 ColorE -> Array2 BGRA32E
convolveDD rad (Array2 kw kh kernel) (Array2 iw ih img) =
  -- trace ("convolveD" ++ show ((kw,kh),(iw,ih)) ++ "\n") $
  Array2 iw' ih' img'
 where
   iw' = iw+2*rad
   ih' = ih+2*rad
   -- Normalize the kernel sample array by dividing each element by the
   -- sum of the elements.
   invTot = recip (sumLoop kh $ \ j' -> sumLoop kw $ \ i' -> kernel (i',j'))
   img' (x,y) = 
     sumLoopLHInc (-rad) rad $ \ dy ->
       sumLoopLHInc (-rad) rad $ \ dx ->
         toBGRA32 ((invTot * kernel (rad-dx,rad-dy)) *^ imgSafe (x-rad+dx,y-rad+dy))
   inBounds (i, j) = 0 <=* i &&* i <* iw &&* 0 <=* j &&* j <* ih
   imgSafe = cond inBounds img empty
           


-- Gaussian kernel
gauss sigma xSq = exp (- xSq / (2 * sigma^2)) / (sqrt (2 * pi) * sigma)


--- Testing

c2 = 1.5 *^ grey 0.3