-- Convert various forms to displayers, for flexibility.

module DisplayConvert where

import Exp ( IntE, Exp, FloatE, f2i, i2f, BGR24E )
import Color
import Exp

import Image
import Arrays
import Bitmap
import Display

-- "Displayer" and "toDisplayer" are no longer appropriate names, since
-- the conversion is into an image.  Also, since we're not supporting
-- int-valued images (packed pixels), the method name should be "toImageC".

class Displayable d where
  -- Canonicalize all 'image' functions/values to ImageBCR.  Boolean
  -- argument says whether to add white background May be better (more
  -- general) to take background color argument, but I then don't know
  -- know how to do the region case.
  toDisplayer :: Bool -> d -> ImageBC
   -- type ImageBCR = FloatE{-time-} -> Image BGR24E

{-
instance Displayable ImageBCR where
  toDisplayer _ imb = imb  -- too late to add background

instance Displayable ImageCR where
  toDisplayer whiteBg im = toDisplayer whiteBg (const im :: ImageBCR)
-}

instance Displayable ImageBC where
  toDisplayer whiteBg imb =
    \ t (x,y) -> {-toBGRA32-} (onBg (imb t) (x,y))
   where
     onBg | whiteBg   = (`over` whiteI)
          | otherwise = id

instance Displayable ImageC where
  toDisplayer whiteBg im = toDisplayer whiteBg (const im :: ImageBC)

instance Displayable (FloatE -> Region) where
  toDisplayer whiteBg regb =
    toDisplayer whiteBg (\ t -> crop (regb t) fgIm)
   where
     -- Background/foreground images
     fgIm | whiteBg   = blackI
          | otherwise = whiteI

instance Displayable Region where
  toDisplayer whiteBg reg =
    toDisplayer whiteBg (const reg :: FloatE -> Region)

-- Display image fractions as grey-scale images.  Warning: FractionE is
-- just a synonym for FloatE.  There's no guarantee that the float is in
-- the range [0,1].  (whiteBg doesn't matter here.)
instance Displayable (FloatE -> Image FractionE) where
  toDisplayer whiteBg imb = toDisplayer whiteBg (lift1 (flip fadeC black) . imb)

instance Displayable (Image FractionE) where
  toDisplayer whiteBg imb =
    toDisplayer whiteBg (const imb :: FloatE -> Image FractionE)

-- Discrete in space 
type DImage  = (IntE, IntE) -> ColorE

-- and in time (MS)
type DImageB = IntE -> DImage

instance Displayable DImageB where
  toDisplayer _ dimb = (\ t (x,y) -> dimb (f2i (t * 1000)) (f2i x, f2i y))

instance Displayable (FloatE -> DImage) where
  toDisplayer _ dimb = (\ t (x,y) -> dimb (t * 1000) (f2i x, f2i y))

instance Displayable DImage where
  toDisplayer _ im = (\ _ (x,y) -> im (f2i x, f2i y))

-- General form for static image-like types.  But overlaps with everything.
-- instance Displayable (FloatE -> d) => Displayable d where
--   toDisplayer im = (disp, False)
--    where
--      (disp,_) = toDisplayer (const im)

